﻿using ThinkGeo.Core;

namespace CheckThinkGeoProjection
{
    internal abstract class CheckOverProjection
    {
        //The PROJ strings is come from GDAL latest version
        private const string PROJ_4326 = "+proj=longlat +datum=WGS84 +no_defs";
        

        public void Check()
        {
            ProjectionConverterWithCode();
            ProjectionConverterWithProjString();
            UnmanagedProjectionConverterWithCode();
            UnmanagedProjectionConverterWithProjString();
        }

        protected abstract string TargetPROJ { get; }
        protected abstract int TargetEPSGCode { get; }
        protected abstract PointShape PointInWGS { get; }
        protected abstract PointShape ExpectedProjectedPoint { get; }

        private void ProjectionConverterWithCode()
        {
            try
            {
                var projectionConverter = new ProjectionConverter(4326, TargetEPSGCode);
                projectionConverter.Open();
                var projectedPoint = projectionConverter.ConvertToExternalProjection(PointInWGS);
                var distanceDiff = ExpectedProjectedPoint.GetDistanceTo(projectedPoint, GeographyUnit.Meter, DistanceUnit.Meter);
                if (distanceDiff > 2)
                {
                    Console.WriteLine($"Error in ProjectionConverter when using EPSG code. Distance Difference= {distanceDiff}");
                }
                else
                {
                    Console.WriteLine("Success in ProjectionConverter when using EPSG code.");
                }

            }
            catch (Exception)
            {
                Console.WriteLine($"Exception in creating ProjectionConverter when using EPSG code.");
            }

        }

        private void ProjectionConverterWithProjString()
        {
            try
            {
                var projectionConverter = new ProjectionConverter(PROJ_4326, TargetPROJ);
                projectionConverter.Open();
                var projectedPoint = projectionConverter.ConvertToExternalProjection(PointInWGS);
                var distanceDiff = ExpectedProjectedPoint.GetDistanceTo(projectedPoint, GeographyUnit.Meter, DistanceUnit.Meter);
                if (distanceDiff > 2)
                {
                    Console.WriteLine($"Error in ProjectionConverter when using PROJ string. Distance Difference= {distanceDiff}");
                }
                else
                {
                    Console.WriteLine("Success in ProjectionConverter when PROJ string.");
                }
            }
            catch (Exception)
            {
                Console.WriteLine($"Exception in creating ProjectionConverter when using PROJ string.");
            }
        }

        private void UnmanagedProjectionConverterWithCode()
        {
            try
            {
                var projectionConverter = new GdalProjectionConverter(4326, TargetEPSGCode);
                projectionConverter.Open();
                var projectedPoint = projectionConverter.ConvertToExternalProjection(PointInWGS);
                var distanceDiff = ExpectedProjectedPoint.GetDistanceTo(projectedPoint, GeographyUnit.Meter, DistanceUnit.Meter);
                if (distanceDiff > 2)
                {
                    Console.WriteLine($"Error in UnmanagedProjectionConverter when using EPSG code. Distance Difference= {distanceDiff}");
                }
                else
                {
                    Console.WriteLine("Success in UnmanagedProjectionConverter when using EPSG code.");
                }
            }
            catch (Exception)
            {
                Console.WriteLine($"Exception in creating UnmanagedProjectionConverter when using EPSG code.");
            }
        }

        private void UnmanagedProjectionConverterWithProjString()
        {
            try
            {
                var projectionConverter = new GdalProjectionConverter(PROJ_4326, TargetPROJ);
                projectionConverter.Open();
                var projectedPoint = projectionConverter.ConvertToExternalProjection(PointInWGS);
                var distanceDiff = ExpectedProjectedPoint.GetDistanceTo(projectedPoint, GeographyUnit.Meter, DistanceUnit.Meter);
                if (distanceDiff > 2)
                {
                    Console.WriteLine($"Error in UnmanagedProjectionConverter when using PROJ string. Distance Difference= {distanceDiff}");
                }
                else
                {
                    Console.WriteLine("Success in UnmanagedProjectionConverter when using PROJ string.");
                }
            }
            catch (Exception)
            {
                Console.WriteLine($"Exception in creating UnmanagedProjectionConverter when using PROJ string.");
            }
        }
    }
}
